/*
 * Copyright (C) 2001, 2002 The Mir-coders group
 *
 * This file is part of Mir.
 *
 * Mir is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Mir is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Mir; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * In addition, as a special exception, The Mir-coders gives permission to link
 * the code of this program with  any library licensed under the Apache Software License,
 * The Sun (tm) Java Advanced Imaging library (JAI), The Sun JIMI library
 * (or with modified versions of the above that use the same license as the above),
 * and distribute linked combinations including the two.  You must obey the
 * GNU General Public License in all respects for all of the code used other than
 * the above mentioned libraries.  If you modify this file, you may extend this
 * exception to your version of the file, but you are not obligated to do so.
 * If you do not wish to do so, delete this exception statement from your version.
 */

package mircoders.localizer.basic;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import mir.bundle.Bundle;
import mir.config.MirPropertiesConfiguration;
import mir.entity.Entity;
import mir.entity.adapter.EntityAdapter;
import mir.entity.adapter.EntityAdapterFailure;
import mir.generator.Generator;
import mir.generator.GeneratorExc;
import mir.log.LoggerWrapper;
import mir.misc.StringUtil;
import mir.module.ModuleExc;
import mir.storage.DatabaseFailure;
import mir.util.DateTimeRoutines;
import mir.util.EntityUtility;
import mir.util.StringRoutines;
import mir.util.generator.BundleGeneratorFunction;
import mircoders.entity.EntityComment;
import mircoders.entity.EntityContent;
import mircoders.global.MirGlobal;
import mircoders.localizer.MirAdminInterfaceLocalizer;
import mircoders.localizer.MirLocalizerExc;
import mircoders.localizer.MirLocalizerFailure;
import mircoders.module.ModuleLanguage;
import mircoders.storage.DatabaseContent;


public class MirBasicAdminInterfaceLocalizer implements MirAdminInterfaceLocalizer {
  /** preview page name (String) -> generator name (String) */
  private Map previewPages;
  private List previewPageNames;

  private List simpleCommentOperations;
  private List simpleArticleOperations;
  private Map simpleCommentOperationsMap;
  private Map simpleArticleOperationsMap;

  protected static LoggerWrapper logger = new LoggerWrapper("Localizer.AdminInterface");
  protected MirPropertiesConfiguration configuration = MirPropertiesConfiguration.instance();

  public MirBasicAdminInterfaceLocalizer() throws MirLocalizerFailure {
    simpleCommentOperations = new ArrayList();
    simpleArticleOperations = new ArrayList();
    simpleCommentOperationsMap = new HashMap();
    simpleArticleOperationsMap = new HashMap();

    previewPages = new HashMap();
    previewPageNames = new ArrayList();

    String[] previewConfiguration = configuration.getStringArray("Mir.Localizer.Admin.ArticlePreview");

    for (int i = 0; i<previewConfiguration.length; i++) {
      try {
        List parts = StringRoutines.separateString(previewConfiguration[i], "=");
        if (parts.size() == 2) {
          String key = ((String) parts.get(0)).trim();
          previewPages.put(key,((String) parts.get(1)).trim());
          previewPageNames.add(key);
        }
        else {
          throw new Exception("'=' expected");
        }
      }
      catch (Exception e) {
        logger.error("Error while processing preview configuration '" + previewConfiguration[i] + "': " + e.toString());
      }
    }

    addSimpleArticleOperation(new ChangeArticleFieldOperation("newswire", "to_article_type", "0", "1", false));
    addSimpleArticleOperation(new ModifyArticleFieldOperation("unhide", "is_published", "1", false));
    addSimpleArticleOperation(new ModifyArticleFieldOperation("hide", "is_published", "0", false));

    addSimpleCommentOperation(new ModifyCommentFieldOperation("unhide", "is_published", "1"));
    addSimpleCommentOperation(new ModifyCommentFieldOperation("hide", "is_published", "0"));
  }

  /** {@inheritDoc} */
  public String getAdminPageGenerator(String aPage, Map aTemplateData, EntityAdapter aUser, String aDefault) throws MirLocalizerExc {
    return aDefault;
  }

  /** {@inheritDoc} */
  public Generator prepareArticlePreview(String aPreviewPage, EntityAdapter anArticle, Map aContext) throws MirLocalizerExc {
    MirGlobal.localizer().producerAssistant().initializeGenerationValueSet(aContext);
    aContext.put("article", anArticle);

    try {
      aContext.put(
          "language",
          MirGlobal.localizer().dataModel().adapterModel().makeEntityAdapter("language",
              new ModuleLanguage().languageForCode(configuration.getString("Mir.Localizer.Admin.ArticlePreviewLanguage"))));
    }
    catch (EntityAdapterFailure e) {
      throw new MirLocalizerFailure("Can't create entity adapter", e);
    }
    catch (ModuleExc e) {
      throw new MirLocalizerFailure("Can't find language", e);
    }
    aContext.put("pathprefix", configuration.getString("Mir.Localizer.Admin.ArticlePreviewLanguage")); 

    aContext.put("lang",
        new BundleGeneratorFunction(
            new Bundle[] {
              MirGlobal.getBundleFactory().getBundle(
                  "etc/bundles/producer",
                  new String[] {
                    configuration.getString("Mir.Localizer.Admin.ArticlePreviewLanguage") }) }));


    if (previewPages.containsKey(aPreviewPage)) {
      try {
        return MirGlobal.localizer().generators().makeAdminGeneratorLibrary().
            makeGenerator((String) previewPages.get(aPreviewPage), null);
      }
      catch (GeneratorExc e) {
        throw new MirLocalizerFailure(e);
      }
    }
		throw new MirLocalizerExc("Unknown preview: " + aPreviewPage);
  }

  /** {@inheritDoc} */
  public List getPreviewPages(EntityAdapter anArticle) throws MirLocalizerExc{
    return Collections.unmodifiableList(previewPageNames);
  }

  /** {@inheritDoc} */
  public String makePasswordDigest(String aPassword) {
    

    return aPassword;
  }

  /** {@inheritDoc} */
  public void initializeArticle(Map anArticle) {
    anArticle.put("is_published", "0");
    anArticle.put("date", StringUtil.date2webdbDate(new GregorianCalendar()));
  }

  public List simpleCommentOperations() {
    return Collections.unmodifiableList(simpleCommentOperations);
  }

  public List simpleArticleOperations() {
    return Collections.unmodifiableList(simpleArticleOperations);
  }

  public MirSimpleEntityOperation simpleArticleOperationForName(String aName) {
    return (MirSimpleEntityOperation) simpleArticleOperationsMap.get(aName);
  }

  public MirSimpleEntityOperation simpleCommentOperationForName(String aName) {
    return (MirSimpleEntityOperation) simpleCommentOperationsMap.get(aName);
  }

  public void removeSimpleArticleOperation(String aName) {
    simpleArticleOperations.remove(simpleArticleOperationsMap.get(aName));
    simpleArticleOperationsMap.remove(aName);
  }

  public void addSimpleArticleOperation(MirSimpleEntityOperation anOperation) {
    removeSimpleArticleOperation(anOperation.getName());
    simpleArticleOperationsMap.put(anOperation.getName(), anOperation);
    simpleArticleOperations.add(anOperation);
  }

  public void removeSimpleCommentOperation(String aName) {
    simpleCommentOperations.remove(simpleCommentOperationsMap.get(aName));
    simpleCommentOperationsMap.remove(aName);
  }

  public void addSimpleCommentOperation(MirSimpleEntityOperation anOperation) {
    removeSimpleCommentOperation(anOperation.getName());
    simpleCommentOperationsMap.put(anOperation.getName(), anOperation);
    simpleCommentOperations.add(anOperation);
  }

  protected abstract static class EntityModifyingOperation implements MirSimpleEntityOperation {
    private String name;
    private boolean logOperation;

    protected EntityModifyingOperation(String aName, boolean aLogOperation) {
      name = aName;
      logOperation = aLogOperation;
    }

    public String getName() {
      return name;
    }

    public boolean isAvailable(EntityAdapter anEntity) {
      try {
        Entity entity = anEntity.getEntity();
        return isAvailable(entity);
      }
      catch (Throwable t) {
        return false;
      }
    }

    public void perform(EntityAdapter aUser, EntityAdapter anEntity) throws MirLocalizerExc, MirLocalizerFailure {
      Entity entity = anEntity.getEntity();
      try {
        performModification(aUser, entity);

        if (logOperation) {
          try {
            StringBuffer line = new StringBuffer();

            line.append(DateTimeRoutines.advancedDateFormat(
                MirGlobal.config().getString("Mir.DefaultDateTimeFormat"),
                (new GregorianCalendar()).getTime(),
                MirGlobal.config().getString("Mir.DefaultTimezone")));
            line.append(" ");
            if (aUser != null)
              line.append(aUser.get("login"));
            else
              line.append("unknown");

            line.append(" ");
            line.append(getName());
            EntityUtility.appendLineToField(entity, "comment", line.toString());
          }
          catch (Throwable t) {
            logger.error("Error while trying to log an article operation: " + t.toString());
          }
        }

        entity.update();
      }
      catch (Throwable t) {
        throw new MirLocalizerFailure(t);
      }
    }

    protected abstract boolean isAvailable(Entity anEntity) throws DatabaseFailure ;
    protected abstract void performModification(EntityAdapter aUser, Entity anEntity)  throws MirLocalizerExc, MirLocalizerFailure ;
  }

  public static abstract class CommentModifyingOperation extends EntityModifyingOperation {
    public CommentModifyingOperation(String aName, boolean aLogOperation) {
      super(aName, aLogOperation);
    }

    public CommentModifyingOperation(String aName) {
      this(aName, true);
    }

    protected boolean isAvailable(Entity anEntity) throws DatabaseFailure {
      return anEntity instanceof EntityComment && isAvailable((EntityComment) anEntity);
    }

    protected void performModification(EntityAdapter aUser, Entity anEntity) throws DatabaseFailure {
      performModification(aUser, (EntityComment) anEntity);
      DatabaseContent.getInstance().setUnproduced("id="+anEntity.getFieldValue("to_media"));
    }

    protected abstract boolean isAvailable(EntityComment aComment) throws DatabaseFailure ;
    protected abstract void performModification(EntityAdapter aUser, EntityComment aComment) throws DatabaseFailure ;
  }

  public static abstract class ArticleModifyingOperation extends EntityModifyingOperation {
    public ArticleModifyingOperation(String aName, boolean aLogOperation) {
      super(aName, aLogOperation);
    }

    protected boolean isAvailable(Entity anEntity) throws DatabaseFailure {
      return anEntity instanceof EntityContent && isAvailable((EntityContent) anEntity);
    }

    protected void performModification(EntityAdapter aUser, Entity anEntity) throws DatabaseFailure {
      performModification(aUser, (EntityContent) anEntity);
      anEntity.setFieldValue("is_produced", "0");
    }

    protected abstract boolean isAvailable(EntityContent anArticle) throws DatabaseFailure ;
    protected abstract void performModification(EntityAdapter aUser, EntityContent anArticle) throws DatabaseFailure ;
  }

  protected static class SetCommentFieldOperation extends CommentModifyingOperation {
    private String field;
    private String value;

    public SetCommentFieldOperation(String aName, String aField, String aValue) {
      super(aName);

      field = aField;
      value = aValue;
    }

    protected boolean isAvailable(EntityComment aComment) {
      return true;
    }

    protected void performModification(EntityAdapter aUser, EntityComment aComment) throws DatabaseFailure {
      aComment.setFieldValue(field, value);
    }
  }

  protected static class SetCommentFieldsOperation extends CommentModifyingOperation {
    private Map values;

    public SetCommentFieldsOperation(String aName, String aFields[], String aValues[]) {
      super(aName);

      values = new HashMap();

      for (int i=0; i<aFields.length; i++)
        values.put(aFields[i], aValues[i]);
    }

    protected boolean isAvailable(EntityComment aComment) {
      return true;
    }

    protected void performModification(EntityAdapter aUser, EntityComment aComment) throws DatabaseFailure {
      Iterator i = values.entrySet().iterator();

      while (i.hasNext()) {
        Map.Entry entry = (Map.Entry) i.next();
        aComment.setFieldValue((String) entry.getKey(), (String) entry.getValue());
      }
    }
  }


  protected static class ModifyCommentFieldOperation extends CommentModifyingOperation {
    private String field;
    private String value;

    public ModifyCommentFieldOperation(String aName, String aField, String aValue) {
      super(aName);

      field = aField;
      value = aValue;
    }

    protected boolean isAvailable(EntityComment aComment) {
      return aComment.getFieldValue(field) == null || !aComment.getFieldValue(field).equals(value);
    }

    protected void performModification(EntityAdapter aUser, EntityComment aComment) throws DatabaseFailure {
      aComment.setFieldValue(field, value);
    }
  }

  protected static class SetArticleFieldOperation extends ArticleModifyingOperation {
    private String field;
    private String value;

    public SetArticleFieldOperation(String aName, String aField, String aValue, boolean aLogOperation) {
      super(aName, aLogOperation);

      field = aField;
      value = aValue;
    }

    protected boolean isAvailable(EntityContent anArticle) {
      return true;
    }

    protected void performModification(EntityAdapter aUser, EntityContent anArticle) throws DatabaseFailure {
      anArticle.setFieldValue(field, value);
    }
  }

  protected static class ModifyArticleFieldOperation extends ArticleModifyingOperation {
    private String field;
    private String value;

    public ModifyArticleFieldOperation(String aName, String aField, String aValue, boolean aLogOperation) {
      super(aName, aLogOperation);

      field = aField;
      value = aValue;
    }

    protected boolean isAvailable(EntityContent anArticle) {
      return anArticle.getFieldValue(field) == null || !anArticle.getFieldValue(field).equals(value);
    }

    protected void performModification(EntityAdapter aUser, EntityContent anArticle) throws DatabaseFailure {
      anArticle.setFieldValue(field, value);
    }
  }

  protected static class ChangeArticleFieldOperation extends ArticleModifyingOperation {
    private String field;
    private Set oldValues;
    private String newValue;

    public ChangeArticleFieldOperation(String aName, String aField, String anOldValues[], String aNewValue, boolean aLogOperation) {
      super(aName, aLogOperation);

      field = aField;
      newValue = aNewValue;
      oldValues = new HashSet(Arrays.asList(anOldValues));
    }
    public ChangeArticleFieldOperation(String aName, String aField, String anOldValue, String aNewValue, boolean aLogOperation) {
      this(aName, aField, new String[] {anOldValue}, aNewValue, aLogOperation);
    }

    protected boolean isAvailable(EntityContent anArticle) {
      return anArticle.getFieldValue(field) != null && oldValues.contains(anArticle.getFieldValue(field));
    }

    protected void performModification(EntityAdapter aUser, EntityContent anArticle) throws DatabaseFailure {
      anArticle.setFieldValue(field, newValue);
    }
  }
}
