/*
 * Copyright (C) 2001, 2002 The Mir-coders group
 *
 * This file is part of Mir.
 *
 * Mir is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Mir is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Mir; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * In addition, as a special exception, The Mir-coders gives permission to link
 * the code of this program with  any library licensed under the Apache Software License,
 * The Sun (tm) Java Advanced Imaging library (JAI), The Sun JIMI library
 * (or with modified versions of the above that use the same license as the above),
 * and distribute linked combinations including the two.  You must obey the
 * GNU General Public License in all respects for all of the code used other than
 * the above mentioned libraries.  If you modify this file, you may extend this
 * exception to your version of the file, but you are not obligated to do so.
 * If you do not wish to do so, delete this exception statement from your version.
 */
package mircoders.producer.reader;

import java.io.File;
import java.util.Map;

import mir.producer.ProducerNode;
import mir.producer.reader.DefaultProducerNodeBuilders;
import mir.producer.reader.ProducerConfigExc;
import mir.producer.reader.ProducerNodeBuilder;
import mir.producer.reader.ProducerNodeBuilderLibrary;
import mir.util.xml.XMLParserExc;
import mir.util.xml.XMLReaderTool;
import mircoders.producer.ContentMarkingProducerNode;
import mircoders.producer.ContentModifyingProducerNode;
import mircoders.producer.IndexingProducerNode;
import mircoders.producer.MediaGeneratingProducerNode;
import mircoders.producer.RadicalendarProducerNode;
import mircoders.producer.UnIndexingProducerNode;
import mircoders.producer.ChangeReportingProducerNode;


public class SupplementalProducerNodeBuilders {

  public static void registerBuilders(ProducerNodeBuilderLibrary aBuilderLibrary, File aBasePath) throws ProducerConfigExc {
    aBuilderLibrary.registerBuilder("ModifyContent", ContentModifyingProducerNodeBuilder.class);
    aBuilderLibrary.registerBuilder("MarkContent", ContentMarkingProducerNodeBuilder.class);
    aBuilderLibrary.registerBuilder("GenerateMedia", MediaGeneratingProducerNodeBuilder.class);

    aBuilderLibrary.registerFactory("IndexContent", new ContentIndexingProducerNodeBuilder.factory(aBasePath));
    aBuilderLibrary.registerFactory("UnIndexContent", new ContentUnIndexingProducerNodeBuilder.factory(aBasePath));
    aBuilderLibrary.registerBuilder("Radicalendar", RadicalendarProducerNodeBuilder.class);

    aBuilderLibrary.registerBuilder("ReportChanges", ChangeReportingProducerNodeBuilder.class);
  }

  private final static String MARKER_KEY_ATTRIBUTE = DefaultProducerNodeBuilders.KEY_ATTRIBUTE;
  private final static String[] MARKER_REQUIRED_ATTRIBUTES = {MARKER_KEY_ATTRIBUTE};
  private final static String[] MARKER_OPTIONAL_ATTRIBUTES = {};
  private final static String[] MARKER_SUBNODES = {};

  public static class ContentMarkingProducerNodeBuilder extends DefaultProducerNodeBuilders.AbstractProducerNodeBuilder {

    private String key;

    public ContentMarkingProducerNodeBuilder() {
      super(MARKER_SUBNODES);
    }

    public void setAttributes(Map anAttributes) throws ProducerConfigExc, XMLParserExc {
      XMLReaderTool.checkAttributes(anAttributes, MARKER_REQUIRED_ATTRIBUTES, MARKER_OPTIONAL_ATTRIBUTES);

      key = (String) anAttributes.get(MARKER_KEY_ATTRIBUTE);
    }

    public ProducerNode constructNode() {
      return new ContentMarkingProducerNode(key);
    }
  }

  private final static String INDEXER_KEY_ATTRIBUTE = DefaultProducerNodeBuilders.KEY_ATTRIBUTE;
  private final static String INDEXER_INDEX_ATTRIBUTE = "pathToIndex";
  private final static String[] INDEXER_REQUIRED_ATTRIBUTES = {INDEXER_KEY_ATTRIBUTE, INDEXER_INDEX_ATTRIBUTE};
  private final static String[] INDEXER_OPTIONAL_ATTRIBUTES = {};
  private final static String[] INDEXER_SUBNODES = {};

  public static class ContentIndexingProducerNodeBuilder extends DefaultProducerNodeBuilders.AbstractProducerNodeBuilder {
    private String key;
    private String pathToIndex;
    private File indexBasePath;

    private static class factory implements ProducerNodeBuilderFactory {
      private File sourceBasePath;

      public factory(File aSourceBasePath) {
        sourceBasePath = aSourceBasePath;
      }

      public ProducerNodeBuilder makeBuilder() {
        return new ContentIndexingProducerNodeBuilder(sourceBasePath);
      }
    }

    public ContentIndexingProducerNodeBuilder(File anIndexBasePath) {
      super(INDEXER_SUBNODES);

      indexBasePath = anIndexBasePath;
    }

    public void setAttributes(Map anAttributes) throws ProducerConfigExc, XMLParserExc {
      XMLReaderTool.checkAttributes(anAttributes, INDEXER_REQUIRED_ATTRIBUTES, INDEXER_OPTIONAL_ATTRIBUTES);

      key = (String) anAttributes.get(INDEXER_KEY_ATTRIBUTE);
      pathToIndex = (String) anAttributes.get(INDEXER_INDEX_ATTRIBUTE);
    }

    public ProducerNode constructNode() {
      return new IndexingProducerNode(indexBasePath, key, pathToIndex);
    }
  }

  private final static String UNINDEXER_KEY_ATTRIBUTE = DefaultProducerNodeBuilders.KEY_ATTRIBUTE;
  private final static String UNINDEXER_INDEX_ATTRIBUTE = "pathToIndex";
  private final static String[] UNINDEXER_REQUIRED_ATTRIBUTES = {UNINDEXER_KEY_ATTRIBUTE, UNINDEXER_INDEX_ATTRIBUTE};
  private final static String[] UNINDEXER_OPTIONAL_ATTRIBUTES = {};
  private final static String[] UNINDEXER_SUBNODES = {};

  public static class ContentUnIndexingProducerNodeBuilder extends DefaultProducerNodeBuilders.AbstractProducerNodeBuilder {
    private String key;
    private String pathToIndex;

    private File indexBasePath;

    private static class factory implements ProducerNodeBuilderFactory {
      private File indexBasePath;

      public factory(File aSourceBasePath) {
        indexBasePath = aSourceBasePath;
      }

      public ProducerNodeBuilder makeBuilder() {
        return new ContentUnIndexingProducerNodeBuilder(indexBasePath);
      }
    }

    public ContentUnIndexingProducerNodeBuilder(File anIndexBasePath) {
      super(UNINDEXER_SUBNODES);

      indexBasePath = anIndexBasePath;
    }

    public void setAttributes(Map anAttributes) throws ProducerConfigExc, XMLParserExc {
      XMLReaderTool.checkAttributes(anAttributes, UNINDEXER_REQUIRED_ATTRIBUTES, UNINDEXER_OPTIONAL_ATTRIBUTES);

      key = (String) anAttributes.get(UNINDEXER_KEY_ATTRIBUTE);
      pathToIndex = (String) anAttributes.get(UNINDEXER_INDEX_ATTRIBUTE);
    }

    public ProducerNode constructNode() {
      return new UnIndexingProducerNode(indexBasePath, key, pathToIndex);
    }
  }

  private final static String CONTENT_MODIFIER_KEY_ATTRIBUTE = DefaultProducerNodeBuilders.KEY_ATTRIBUTE;
  private final static String CONTENT_MODIFIER_FIELD_ATTRIBUTE = "field";
  private final static String CONTENT_MODIFIER_VALUE_ATTRIBUTE = "value";
  private final static String[] CONTENT_MODIFIER_REQUIRED_ATTRIBUTES = {CONTENT_MODIFIER_KEY_ATTRIBUTE, CONTENT_MODIFIER_FIELD_ATTRIBUTE, CONTENT_MODIFIER_VALUE_ATTRIBUTE};
  private final static String[] CONTENT_MODIFIER_OPTIONAL_ATTRIBUTES = {};
  private final static String[] CONTENT_MODIFIER_SUBNODES = {};

  public static class ContentModifyingProducerNodeBuilder extends DefaultProducerNodeBuilders.AbstractProducerNodeBuilder {

    private String key;
    private String field;
    private String value;

    public ContentModifyingProducerNodeBuilder() {
      super(CONTENT_MODIFIER_SUBNODES);
    }

    public void setAttributes(Map anAttributes) throws ProducerConfigExc, XMLParserExc {
      XMLReaderTool.checkAttributes(anAttributes, CONTENT_MODIFIER_REQUIRED_ATTRIBUTES, CONTENT_MODIFIER_OPTIONAL_ATTRIBUTES);

      key = (String) anAttributes.get(CONTENT_MODIFIER_KEY_ATTRIBUTE);
      field = (String) anAttributes.get(CONTENT_MODIFIER_FIELD_ATTRIBUTE);
      value = (String) anAttributes.get(CONTENT_MODIFIER_VALUE_ATTRIBUTE);
    }

    public ProducerNode constructNode() {
      return new ContentModifyingProducerNode(key, field, value);
    }
  }

  private final static String MEDIA_KEY_ATTRIBUTE = DefaultProducerNodeBuilders.KEY_ATTRIBUTE;
  private final static String[] MEDIA_REQUIRED_ATTRIBUTES = {MEDIA_KEY_ATTRIBUTE};
  private final static String[] MEDIA_OPTIONAL_ATTRIBUTES = {};
  private final static String[] MEDIA_SUBNODES = {};

  public static class MediaGeneratingProducerNodeBuilder extends DefaultProducerNodeBuilders.AbstractProducerNodeBuilder {

    private String key;

    public MediaGeneratingProducerNodeBuilder() {
      super(MEDIA_SUBNODES);
    }

    public void setAttributes(Map anAttributes) throws ProducerConfigExc, XMLParserExc {
      XMLReaderTool.checkAttributes(anAttributes, MEDIA_REQUIRED_ATTRIBUTES, MEDIA_OPTIONAL_ATTRIBUTES);

      key = (String) anAttributes.get(MEDIA_KEY_ATTRIBUTE);
    }

    public ProducerNode constructNode() {
      return new MediaGeneratingProducerNode(key);
    }
  }

  public static class RadicalendarProducerNodeBuilder extends DefaultProducerNodeBuilders.AbstractProducerNodeBuilder {
    private final static String RC_KEY_ATTRIBUTE = "key";
    private final static String RC_GROUPNAME_ATTRIBUTE = "groupname";
    private final static String RC_GMTOFFSET_ATTRIBUTE = "gmtoffset";
    private final static String[] RC_REQUIRED_ATTRIBUTES = {RC_KEY_ATTRIBUTE, RC_GROUPNAME_ATTRIBUTE, RC_GMTOFFSET_ATTRIBUTE};
    private final static String[] RC_OPTIONAL_ATTRIBUTES = {};
    private final static String[] RC_SUBNODES = {};
    private String key;
    private String groupname;
    private String gmtoffset;

    public RadicalendarProducerNodeBuilder() {
      super(RC_SUBNODES);
    }

    public void setAttributes(Map anAttributes) throws ProducerConfigExc, XMLParserExc {
      XMLReaderTool.checkAttributes(anAttributes, RC_REQUIRED_ATTRIBUTES, RC_OPTIONAL_ATTRIBUTES);

      key = (String) anAttributes.get(RC_KEY_ATTRIBUTE);
      groupname = (String) anAttributes.get(RC_GROUPNAME_ATTRIBUTE);
      gmtoffset = (String) anAttributes.get(RC_GMTOFFSET_ATTRIBUTE);
    }

    public ProducerNode constructNode() {
      return new RadicalendarProducerNode(key, groupname, gmtoffset);
    }
  }

  public static class ChangeReportingProducerNodeBuilder extends DefaultProducerNodeBuilders.AbstractProducerNodeBuilder {
    private final static String CHANGEREPORTING_FILE_ATTRIBUTE = "file";
    private final static String CHANGEREPORTING_FORMAT_ATTRIBUTE = "format";
    private final static String CHANGEREPORTING_BASEPATH_ATTRIBUTE = "basepath";
    private final static String CHANGEREPORTING_EXCLUDEDPATHS_ATTRIBUTE = "excludedpaths";
    private final static String[] CHANGEREPORTING_REQUIRED_ATTRIBUTES = {CHANGEREPORTING_FILE_ATTRIBUTE,
        CHANGEREPORTING_FORMAT_ATTRIBUTE, CHANGEREPORTING_BASEPATH_ATTRIBUTE};
    private static final String[] CHANGEREPORTING_OPTIONAL_ATTRIBUTES = {CHANGEREPORTING_EXCLUDEDPATHS_ATTRIBUTE};

    private String file;
    private String format;
    private String basepath;
    private String excludedpaths;

    public ChangeReportingProducerNodeBuilder() {
      super(new String[0]);
    }

    public void setAttributes(Map anAttributes) throws ProducerConfigExc, XMLParserExc {
      XMLReaderTool.checkAttributes(anAttributes, CHANGEREPORTING_REQUIRED_ATTRIBUTES, CHANGEREPORTING_OPTIONAL_ATTRIBUTES);

      file = (String) anAttributes.get(CHANGEREPORTING_FILE_ATTRIBUTE);
      format = (String) anAttributes.get(CHANGEREPORTING_FORMAT_ATTRIBUTE);
      basepath = (String) anAttributes.get(CHANGEREPORTING_BASEPATH_ATTRIBUTE);
      excludedpaths = (String) anAttributes.get(CHANGEREPORTING_EXCLUDEDPATHS_ATTRIBUTE);
    }

    public ProducerNode constructNode() {
      return new ChangeReportingProducerNode(file, format, basepath, excludedpaths);
    }
  }
}


